/*
   Copyright (c) 2015 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
*/

#include "main.h"
#include "pwm.h"
#include "stm32f4xx_tim.h"


//structure for pwm configuration mirroring
PWMCONF pwmConf;

void initPWM(PWMGEN *aV)
{
	NVIC_InitTypeDef NVIC_InitStructure;
	uint16_t PrescalerValue = 0;
	
	GPIO_InitTypeDef GPIO_InitStructure;
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
  /* TIM1 clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);
	
	/* Enable the TIM1 global Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = TIM1_CC_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 4;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
	
	//TIM Interrupts enable
  TIM_ITConfig(TIM1, TIM_IT_CC4 , ENABLE);

  /* GPIO clock enable */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB, ENABLE);
  
  // GPIOA Configuration: TIM1: CH1N (PA7), CH1(PA8), CH2(PA9), CH3(PA10)
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	/* Connect TIM1 Port A pins to AF2 */  
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_TIM1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource8, GPIO_AF_TIM1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_TIM1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_TIM1);
	
	// GPIOB Configuration: TIM1: CH2N (PB0), CH3N (PB1)
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_14 | GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
	
	/* Connect TIM1 PORT B pins to AF2 */  
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource0, GPIO_AF_TIM1);
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource1, GPIO_AF_TIM1);
	
	/* Compute the prescaler value */
  PrescalerValue = (uint16_t) ((SystemCoreClock) / 80000000) - 1;
	
  /* Time base configuration */
  //ARR
	TIM_TimeBaseStructure.TIM_Period = aV->PeriodMax;
	TIM_TimeBaseStructure.TIM_Prescaler = PrescalerValue;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  //CMS
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

	//OCxPE
	TIM_OC1PreloadConfig(TIM1, TIM_OCPreload_Enable);
	TIM_OC2PreloadConfig(TIM1, TIM_OCPreload_Enable);
	TIM_OC3PreloadConfig(TIM1, TIM_OCPreload_Enable);
	TIM_OC4PreloadConfig(TIM1, TIM_OCPreload_Enable);
	
  //ARPE
	TIM_ARRPreloadConfig(TIM1, ENABLE);
	
	//TIM1 enable counter 
	//CEN
  TIM_Cmd(TIM1, ENABLE);
}

/*
* Stop PWM/Counter and Reset to initial state
*/
void ResetPWM()
{
	//Stop Counter Operation
	TIM_Cmd(TIM1, DISABLE);
	//Reset Counter Value
	TIM1->CNT = 0;	
}
/*
* Update PWM configuration mirror with actual settings
*/
void updateConfigMirror()
{
   //For timer 1
	 pwmConf.ARR = TIM1->ARR;
	 pwmConf.CCR1 = TIM1->CCR1;
	 pwmConf.OC1M = (TIM1->CCMR1 >> 4) & 7;
	 pwmConf.CCR2 = TIM1->CCR2;
	 pwmConf.OC2M = (TIM1->CCMR1 >> 12) & 7;
	 pwmConf.CCR3 = TIM1->CCR3;
	 pwmConf.OC3M = (TIM1->CCMR2 >> 4) & 7;
	 pwmConf.CCR4 = TIM1->CCR4;
	 pwmConf.OC4M = (TIM1->CCMR2 >> 12) & 7;
	 pwmConf.CCER = TIM1->CCER;
}
/*
 * Update the configuration of the used pwm modules
 */
void updatePWM(PWMGEN *aV)
{
   uint32_t Tmp, Tmp2;
   int16_t Period, GPR0_BLDC_PWM;
   int16_t CMPB_SMPL_POINT;

   //Convert "Period" (Q15) modulation function to Q0
   Tmp = (int32_t) aV->PeriodMax * (int32_t) aV->MfuncPeriod; //Q15 = Q0xQ15
   Period = (int16_t) (Tmp >> 15);                        //Q15 -> Q0 (Period)

   // Check PwmActive setting
   if (aV->PwmActive == 1)
   {
      //PWM active high
      GPR0_BLDC_PWM = 0x7FFF - aV->DutyFunc;
      CMPB_SMPL_POINT = 0x7FFF - (aV->DutyFunc >> 1);    //Sample in center of PWM pulse using CMPB
   }
   else if (aV->PwmActive == 0)
   {
      GPR0_BLDC_PWM = aV->DutyFunc;
      CMPB_SMPL_POINT = aV->DutyFunc >> 1;               //Sample in center of PWM pulse using CMPB
   }

   //Convert "DutyFunc" or "GPR0_BLDC_PWM" (Q15) duty modulation function to Q0
   Tmp = (int32_t) Period * (int32_t) GPR0_BLDC_PWM;        //Q15 = Q0xQ15
   Tmp2 = (int32_t) Period * (int32_t) CMPB_SMPL_POINT;     //Q15 = Q0xQ15
   //ADC Trigger
	 TIM1->CCR4 = (uint16_t) (Tmp2 >> 15); //Sample in center of PWM pulse
	 
   //Copy CCER,CCMR1 and CCMR2 for Bit Operations
	 uint16_t CCERtmp = TIM1->CCER;
	 uint16_t CCMR1tmp = TIM1->CCMR1;
	 uint16_t CCMR2tmp = TIM1->CCMR2;
	 
	 //Clear all relevant bits
	 CCERtmp &= 0xEAAA;
	 CCMR1tmp &= 0x8F8F;
	 CCMR2tmp &= 0x8F8F;
	 
	 //State s1: current from phase A->B, C de-energized
   if (aV->CmtnPointer == 0)
   {
      //Phase 1: OC1 pulsed(PWM1 Mode), OC1N disabled -> 0
			CCERtmp |= 0x1;
			TIM1->CCR1 = (uint16_t) (Tmp >> 15);
		  CCMR1tmp |= TIM_OCMode_PWM2;
			
		  //Phase 2: OC2 Forced to 0, OC2N Forced to 1
		  CCERtmp |= 0x5 << 4;
			TIM1->CCR2 = 0;
			CCMR1tmp |= (uint32_t)0x0040 << 8;
		  
		  //Phase 3: OC3 Forced to 0, OC3N disabled -> 0
			CCERtmp |= 0x0 << 8;
		  TIM1->CCR3 = 0;
			CCMR2tmp |= (uint32_t)0x0040;
	 }

   //State s2: current from phase A->C, B de-energized
   else if (aV->CmtnPointer == 1)
   {
      //Phase 1: OC1 pulsed(PWM1 Mode), OC1N disabled -> 0
			CCERtmp |= 0x1;
			TIM1->CCR1 = (uint16_t) (Tmp >> 15);
		  CCMR1tmp |= TIM_OCMode_PWM2;
			
		  //Phase 2: OC2 Forced to 0, OC2N disabled -> 0 
		  CCERtmp |= 0x0 << 4;
			TIM1->CCR2 = 0;
			CCMR1tmp |= (uint32_t)0x0040 << 8;
		  
		  //Phase 3: OC3 Forced to 0, OC3N Forced to 1
			CCERtmp |= 0x5 << 8;
		  TIM1->CCR3 = 0;
			CCMR2tmp |= (uint32_t)0x0040;
   }

   //State s3: current from phase B->C, A de-energized
   else if (aV->CmtnPointer == 2)
   {
      //Phase 1: OC1 Forced to 0, OC1N disabled -> 0 
			CCERtmp |= 0x0;
			TIM1->CCR1 = 0;
		  CCMR1tmp |= (uint32_t)0x0040;
			
		  //Phase 2: OC2 pulsed(PWM1 Mode), OC2N disabled -> 0 
		  CCERtmp |= 0x1 << 4;
			TIM1->CCR2 = (uint16_t) (Tmp >> 15);
			CCMR1tmp |= TIM_OCMode_PWM2 << 8;
		  
		  //Phase 3: OC3 Forced to 0, OC3N Forced to 1
			CCERtmp |= 0x5 << 8;
		  TIM1->CCR3 = 0;
			CCMR2tmp |= (uint32_t)0x0040;
   }

   //State s4: current from phase B->A, C de-energized
   else if (aV->CmtnPointer == 3)
   {
     //Phase 1: OC1 Forced to 0, OC1N Forced to 1 
			CCERtmp |= 0x5;
			TIM1->CCR1 = 0;
		  CCMR1tmp |= (uint32_t)0x0040;
			
		  //Phase 2: OC2 pulsed(PWM1 Mode), OC2N disabled -> 0 
		  CCERtmp |= 0x1 << 4;
			TIM1->CCR2 = (uint16_t) (Tmp >> 15);
			CCMR1tmp |= TIM_OCMode_PWM2 << 8;
		  
		  //Phase 3: OC3 Forced to 0, OC3N disabled -> 0 
			CCERtmp |= 0x0 << 8;
		  TIM1->CCR3 = 0;
			CCMR2tmp |= (uint32_t)0x0040;
   }

   //State s5: current from phase C->A, B de-energized
   else if (aV->CmtnPointer == 4)
   {
      //Phase 1: OC1 Forced to 0, OC1N Forced to 1
			CCERtmp |= 0x5;
			TIM1->CCR1 = 0;
		  CCMR1tmp |= (uint32_t)0x0040;
			
		  //Phase 2: OC2 Forced to 0, OC2N disabled -> 0 
		  CCERtmp |= 0x0 << 4;
			TIM1->CCR2 = 0;
			CCMR1tmp |= (uint32_t)0x0040 << 8;
		  
		  //Phase 3: OC3 pulsed(PWM1 Mode), OC3N disabled -> 0 
			CCERtmp |= 0x1 << 8;
		  TIM1->CCR3 = (uint16_t) (Tmp >> 15);
			CCMR2tmp |= TIM_OCMode_PWM2;
   }

   //State s6: current from phase C->B, A de-energized
   else if (aV->CmtnPointer == 5)
   {
			//Phase 1: OC1 Forced to 0, OC1N disabled -> 0 
			CCERtmp |= 0x0;
			TIM1->CCR1 = 0;
		  CCMR1tmp |= (uint32_t)0x0040;
			
		  //Phase 2: OC2 Forced to 0, OC2N Forced to 1
		  CCERtmp |= 0x5 << 4;
			TIM1->CCR2 = 0;
			CCMR1tmp |= (uint32_t)0x0040 << 8;
		  
		  //Phase 3: OC3 pulsed(PWM1 Mode), OC3N disabled -> 0 
			CCERtmp |= 0x1 << 8;
		  TIM1->CCR3 = (uint16_t) (Tmp >> 15);
			CCMR2tmp |= TIM_OCMode_PWM2;				
   }	 
	 //Update Registers
	 TIM1->CCER = CCERtmp;
	 TIM1->CCMR1 = CCMR1tmp;
	 TIM1->CCMR2 = CCMR2tmp;
	 
   //update configuration mirror
   updateConfigMirror();	
}

