/*
   Copyright (c) 2015 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
*/

//Define the structure of the PWM Driver Object
typedef struct
{
   PIL_READ_PROBE(uint16_t,CmtnPointer,0, 1, "");     // Input: Commutation (or switching) state pointer input (Q0)
   int16_t MfuncPeriod;      													// Input: Duty ratio of the PWM outputs (Q15)
   uint16_t PeriodMax;    														// Parameter: Maximum period (Q0)
   int16_t DutyFunc;   	   														// Input: PWM period modulation input (Q15)
   uint16_t PwmActive;       													// Parameter: 0 = PWM active low, 1 = PWM active high (0 or 1)
} PWMGEN;

/*
* The "PwmActive" setting depends on Power devices.
* when PwmActive = 1 (active high) implies the power device turns ON with a HIGH gate signal.
* PwmActive = 0 (active low) implies the power device turns ON with a LOW gate signal.
*/

//Define a structure for PWM_module Configuration mirroring
typedef struct
{
		 //PIL Registers
		 PIL_READ_PROBE(uint16_t,ARR,0, 1, "");		// Period Register	
		 PIL_READ_PROBE(uint16_t,CCR1,0, 1, "");  // Compare 1 value
		 PIL_READ_PROBE(uint16_t,OC1M,0, 1, "");  // Channel 1 Mode
		 PIL_READ_PROBE(uint16_t,CCR2,0, 1, "");  // Compare 2 value
		 PIL_READ_PROBE(uint16_t,OC2M,0, 1, "");  // Channel 2 Mode	 
		 PIL_READ_PROBE(uint16_t,CCR3,0, 1, "");  // Compare 3 value
		 PIL_READ_PROBE(uint16_t,OC3M,0, 1, "");  // Channel 3 Mode
		 PIL_READ_PROBE(uint16_t,CCR4,0, 1, "");  // Compare 4 value
		 PIL_READ_PROBE(uint16_t,OC4M,0, 1, "");  // Channel 4 Mode
		 PIL_READ_PROBE(uint16_t,CCER,0, 1, "");  // CCER		
} PWMCONF;

//Define a PWMGEN_handle
typedef PWMGEN *PWMGEN_handle;

//Define ARR
#define PWMPRD PWMCTRPRD 

//Default Initializers for the F281X PWMGEN Object
#define STM_F4_BLDC_PWM_GEN   {0x0000, \
                                0x7FFF, \
                                PWMPRD, \
                                0x0000, \
                                0x0001, \
                             }
#define PWMGEN_DEFAULTS 	STM_F4_BLDC_PWM_GEN

extern void initPWM(PWMGEN *aV);
extern void updatePWM(PWMGEN *aV);
//#endif
