NumberOfSeriesConnectedPanels = 3;   % [] - number of panels connected in series in the system

%%% Solar Panel (KC200GT Solar Array)

SP.Ns = 54;       % [] - number of solar cells in a string in solar array
SP.Np = 3;        % [] - number of solar cell strings in parallel solar array

SP.T_n = 25;      % [C] - nominal temperature of solar cells
SP.G_n = 1000;    % [W/m^2] - nominal irradiation of solar cells
SP.V_OC_n = 32.9; % [V] - nominal open circuit voltage of array (single panel in system)
SP.I_SC_n = 8.21; % [A] - nominal short circuit current of array

SP.Rs = 0.221;    % [ohm] - equvalent series resistance of array
SP.Rp = 415.405;  % [ohm] - equvalent parallel resistance of array

SP.a = 1.3;       % [] - diode ideality constant

SP.K_V = -0.1230; % [V/K] - thermal voltage coefficient of array
SP.K_I = 0.0032;  % [A/K] - thermal current coefficient of array

SP.C_in = 22e-6;  % [F] - input capacitance of the DC/DC converter

%% Bus Voltage

V_bus_Init = 400;      % [V] - DC bus initial voltage of capacitance
C_bus = 1000e-6;   % [F] - DC bus capacitance
V_bus_desired = 400;   % [V] - DC bus desired voltage of capacitance

%%% DC/DC Converter

DCDC.L = 1e-3;    % [H] - boost stage inductor
DCDC.R = 1e-3;    % [Ohm] - boost stage inductor resistance

DCDC.fsw = 20e3;  % [Hz] - switching frequency of boost converter

DCDC.MinDuty = 0.0; % [] - minimum allowable duty cycle for boost converter
DCDC.MaxDuty = 0.95; % [] - maximum allowable duty cycle for boost converter

%% Current-controller:

% Small time constants:

CC_DCDC.T_sw = 1/DCDC.fsw;  % [s] - switching period
CC_DCDC.T_PWM = 0.5*CC_DCDC.T_sw; % [s] - PWM modulator small time constant

CC_DCDC.T_cs = 2/(2*pi*(50e3)); % [s] - Current sensor small time constant (50 kHz bandwidth)

CC_DCDC.f_samp = DCDC.fsw; % [Hz] - Sampling frequency
CC_DCDC.T_samp = 1/CC_DCDC.f_samp; % [s] - Sampling small time constant

CC_DCDC.T_pe = CC_DCDC.T_PWM + CC_DCDC.T_cs + CC_DCDC.T_samp; % [s] - Equivalent small time constant

% Dominant time constants:
CC_DCDC.Tn = DCDC.L/DCDC.R; % [] - time constant for numerator
CC_DCDC.Ti = (2*CC_DCDC.T_pe)/DCDC.R; % [] - time constant for denominator

% Controller Design:
CC_DCDC.Kp=CC_DCDC.Tn/CC_DCDC.Ti; % [] - Kp gain
CC_DCDC.Ki=1/CC_DCDC.Ti;     % [Hz] - Ki gain
CC_DCDC.K_aw=1/CC_DCDC.Kp;   % [] - anti-windup gain
CC_DCDC.upper=SP.V_OC_n*NumberOfSeriesConnectedPanels;
CC_DCDC.lower=0;

%% Voltage controller (DCDC)
VC_DCDC.Tp_Vloop = 10/DCDC.fsw;
VC_DCDC.Tsp=VC_DCDC.Tp_Vloop;
VC_DCDC.Ti_Vloop = 8/SP.C_in*VC_DCDC.Tp_Vloop*VC_DCDC.Tp_Vloop;
VC_DCDC.Tn_Vloop = 4*VC_DCDC.Tp_Vloop;

VC_DCDC.Kp = VC_DCDC.Tn_Vloop/VC_DCDC.Ti_Vloop;
VC_DCDC.Ki = 1/VC_DCDC.Ti_Vloop;
VC_DCDC.K_aw=1/VC_DCDC.Kp;
VC_DCDC.upper=30;
VC_DCDC.lower=0;

%% MPPT Controller:

DCDC.T_samp = 1/100; % MPPT execution time

% Global Search

Global.searchThreshold = 0.1;  % [] - change in power generation of 10% enables global search
Global.VRefMin = SP.V_OC_n * NumberOfSeriesConnectedPanels * 0.1;  % [] - set minimum reference voltage for global search at 10% of system VOC
Global.VRefMax = SP.V_OC_n * NumberOfSeriesConnectedPanels * 0.95;  % [] - set minimum reference voltage for global search at 95% of system VOC
Global.numGlobalSearchZones = 20;  % [] - number of voltage zones during global search sweep
Global.numOfStabilizationCycles = 10;  % [] - number of counts before another global search can be performed

% Incremental Conductance Method (INC):

INC.VStopLimit = 1e-4; % [V] - stopping criteria for voltage change
INC.IStopLimit = 1e-3; % [A] - stopping criteria for current change
INC.PStopLimit = 1e-3; % [W] - stopping criteria for power change

INC.VrefStepMax = 3*SP.V_OC_n * 0.15; % [V] - maximum step change in voltage

INC.dPdVMax = 80; % [W/V] - maximum slope of change in power per unit change in voltage

INC.Alpha = INC.VrefStepMax/INC.dPdVMax; % [V^2/W] - acceleration factor
INC.Beta = 0.1; % [V/A] - acceleration factor

% Perturb and Observe Method:

PnO.VrefStepMin = 1;  % [V] - minimum step change in voltage
PnO.dPdVMax = 15;     % [W/V] - maximum slope of change in power per unit change in voltage
PnO.dPdVMin = -10;    % [W/V] - minimum slope of change in power per unit change in voltage
PnO.VrefChangeFactor = 0.25; % [] - acceleration factor

%%% DC/AC Converter
%% Grid-side:

Grid.Vs = sqrt(2)*230/2; % [V] - amplitude
Grid.f_grid = 50; % [Hz] - grid frequency

%% VSI grid filter:

Grid.L = 5e-3; % [H] - grid-side filter inductance
Grid.R = 0.15; % [Ohm] - grid-side filter resistance

%% Inverter:
DCAC.f_sw = 20e3; % [Hz] - switching frequency

%% Current-controller:

% Small time constants:

CC.T_sw = 1/DCAC.f_sw;  % [s] - switching period
CC.T_PWM = 0.5*CC.T_sw; % [s] - PWM modulator small time constant

CC.T_cs = 2/(2*pi*(50e3)); % [s] - Current sensor small time constant (50 kHz bandwidth)

CC.f_samp = DCAC.f_sw; % [Hz] - Sampling frequency
CC.T_samp = 1/CC.f_samp; % [s] - Sampling small time constant

CC.T_pe = CC.T_PWM + CC.T_cs + CC.T_samp; % [s] - Equivalent small time constant

% Dominant time constants:
CC.Tn = Grid.L/Grid.R; % [] - time constant for numerator
CC.Ti = (2*CC.T_pe)/Grid.R; % [] - time constant for denominator

% Controller Design:
CC.Kp=CC.Tn/CC.Ti; % [] - Kp gain
CC.Ki=1/CC.Ti;     % [Hz] - Ki gain
CC.K_aw=1/CC.Kp;   % [] - anti-windup gain

%% Voltage controller
VC.Tp_Vloop = 1/Grid.f_grid/4;
VC.Tsp=VC.Tp_Vloop;
VC.Ti_Vloop = 8/C_bus*VC.Tp_Vloop*VC.Tp_Vloop;
VC.Tn_Vloop = 4*VC.Tp_Vloop;

VC.Kp = VC.Tn_Vloop/VC.Ti_Vloop;
VC.Ki = 1/VC.Ti_Vloop;
VC.K_aw=1/VC.Kp;
VC.upper=25;
VC.lower=0;

%% PLL
PLL.T_samp = 1/10e3;
PLL.PI_Tn = 1;
PLL.PI_Ti = 2/1000;
PLL.SOGI_Gain = 0.8;