%% Grid
Vin = 120*sqrt(2); % Grid voltage (V)
f = 50.0; % Grid frequency (Hz)

%% AC/DC Converter Design
C = 360e-6; % Output capacitor (C)

% AC/DC Choke
L = 700e-6; % Choke inductance (H)
Rl = 0.001; % Choke resistance (ohms)

fs = 100e3; % Switching frequency (Hz)

% Design of Compensators
Vmax = 440.0; % Maximum voltage (V) %FOR WHAT
Imax = 12.54; % Maximum current (A) %FOR WHAT

Rmax = Vmax/Imax; % Maximum load impedance (ohms)

f_Vloop = 2e3; % Voltage control loop frequency (Hz)
BW_Vloop = 10; % Voltage loop bandwidth (Hz)
IBW_Vloop = 2.5; % Integral voltage loop bandwidth (Hz)

f_Iloop = 50e3; % Current control loop frequency (Hz)
BW_Iloop = 4e3; % Current loop bandwidth (Hz)
IBW_Iloop = 1e3; % Integral current loop bandwidth (Hz)

f_LBloop = 2e3; % Load balancing control loop frequency (Hz)
BW_LBloop = 200; % Load balancing loop bandwidth (Hz)
IBW_LBloop = 50; % Integral balancing loop bandwidth (Hz)

f_FFloop = 50e3; % Voltage feedforward loop frequency (Hz)
f_AVG = 100e3; % Average voltage calculation frequency (Hz)

% Current Compensator
Tp_Iloop = (1.0/fs)/20.0; % Average PWM delay (s)
Ti_Iloop = 2/Rl*Tp_Iloop; %FOR WHAT
Tn_Iloop = L/Rl; %FOR WHAT
Ra = (Tn_Iloop - (1/f_Iloop)/2)/Ti_Iloop; %FOR WHAT
Rsa = (1/f_Iloop)/Ti_Iloop; %FOR WHAT
Ra = Ra*0.05; % - tweak %FOR WHAT

% Voltage Compensator
Ka = 2*pi*L*BW_LBloop/Rmax; %FOR WHAT
Ksa = 2*pi*Ka*IBW_LBloop/f_LBloop; %FOR WHAT

%% Bus
C_bus = 1e-6; % Bus capacitance (F)
Vdc_tar = 300; % Target bus voltage (VDC)

%% DC/DC Converter Design

V_o_max = Vmax; % Maximum output voltage target (VDC)
V_o_min = 50; % Minimum output voltage target (VDC)

Vo_tar = 120; %FOR WHAT

% Transformer
N_p = 50; % Number of turns on primary side windings
N_s = 150; % Number of turns on secondary side windings

mu0 = 4*pi*1e-7; % Magnetic constant/permeability of free space (N/A^2)
mu_r = 6500; % Unsaturated relative permeability (H/m)

Ae_side = 360e-6*2; % Cross-sectional area of core side legs (m^2)
Ae_mid = 704e-6*2; % Cross-sectional area of core middle leg (m^2)
% (2 cores are in parallel)

Le_side = 57.2e-3; % Length of flux path of side core legs (m)
Le_mid = 26e-3; % Length of flux path of middle core leg (m)
Le_gap = 3e-3; % Length of flux path of all leg air gaps (m)

Rm1 = 1/(mu0*mu_r)*Le_side*2/(Ae_side) + 1/mu0*Le_gap/(Ae_side); % Reluctance of side leg of the core %FOR WHAT
Rm2 = 1/(mu0*mu_r)*Le_mid*2/(Ae_mid) + 1/mu0*Le_gap/(Ae_mid); % Reluctance of middle leg of the core %FOR WHAT
Rm3 = Rm1; % Reluctance of side leg of the core %FOR WHAT

M = N_s^2*Rm2/(Rm1*Rm2 + Rm1*Rm3 + Rm2*Rm3); %FOR WHAT
L1 = N_s^2*(Rm2+Rm3)/(Rm1*Rm2 + Rm1*Rm3 + Rm2*Rm3) - M; %FOR WHAT
L2 = N_s^2*(Rm1+Rm2)/(Rm1*Rm2 + Rm1*Rm3 + Rm2*Rm3) - M; %FOR WHAT

Lm_delta = (L1*L2 + L1*M + L2*M)/M*(N_p/N_s)^2; % Equivalent magnetization inductance at primary side (H)
L1_delta = (L1*L2 + L1*M + L2*M)/L2; % Equivalent output series inductance L1 at secondary side (H)
L2_delta = (L1*L2 + L1*M + L2*M)/L1; % Equivalent output series inductance L2 at secondary side (H)

L_o = L1_delta; % Used in model gain controller parameter (H)

% Output capacitor
C_o = 1000e-6; % (C)

% Load
P_n = 1.4e3; % Nominal output power (W)
R_o = Vo_tar^2/P_n; % Output resistance at half rated power (ohms)

% Modulator
f_sw = 20e3; % Switching frequency (Hz)

% Transformer winding resistance taking skin effect into account
h_p_copper = 0.1e-3; %FOR WHAT
h_p_total = 0.18e-3; %FOR WHAT
h_s_copper = 0.3e-3; %FOR WHAT
h_s_total = 0.38e-3; %FOR WHAT
sigma = 5.96e7; %FOR WHAT
v_p = h_p_copper*sqrt(pi*mu0*sigma*f_sw); %FOR WHAT
v_s = h_s_copper*sqrt(pi*mu0*sigma*f_sw); %FOR WHAT
k_skin_p = v_p*(sinh(2*v_p) + sin(2*v_p))/(cosh(2*v_p)-cos(2*v_p)); %FOR WHAT
k_skin_s = v_s*(sinh(2*v_s) + sin(2*v_s))/(cosh(2*v_s)-cos(2*v_s)); %FOR WHAT
l_p = ((22+32*2)*1e-3*2*2+ 2*h_p_total*4*(N_p-1))*N_p/2; %FOR WHAT
s_p = 36.5e-3*h_p_copper; %FOR WHAT
l_s = (((22+32*2)*1e-3*2 + 2*h_p_total*4*N_p)*2 + 2*h_s_total*4*(N_s-1))*N_s/2; %FOR WHAT
s_s = 36.5e-3*h_s_copper; %FOR WHAT
R_p = 1.68e-8*l_p/s_p*k_skin_p; % Primary side equivalent resistance (ohms)
R_s = 1.68e-8*l_s/s_s*k_skin_s; % Secondary side equivalent resistance (ohms)

% Series inductance of the transformer
L_r = 3e-6; % (H)

% Equivalent secondary winding parameters
L_s_eqv = (L1_delta * L2_delta)/(L1_delta + L2_delta); % Equivalent inductance (H)
R_s_eqv = R_s * R_s/(R_s + R_s); % Equivalent resistance (ohms)

%% Equivalent Parameter for controller design
DCDC.L = L_s_eqv + L_r*N_p/N_s; % Output inductances (H)
DCDC.Lr= R_s_eqv + R_p*N_p/N_s; % Inductor resistance (ohms)
DCDC.C = C_o; % Output capacitance (C)
DCDC.fs = f_sw; % Switching frequency (Hz)
DCDC.Ts = 1/DCDC.fs; % Switching period (s)
DCDC.Vamp = V_o_max; % Peak voltage amplitude (V)

%% Current Controls

% Small time constants
CC.PWM = 0.5*DCDC.Ts; % PWM modulator small time constant (s)
CC.Tcs = 1/(2*pi*(10*DCDC.fs)); % Current sensor small time constant (s)
CC.f_samp = DCDC.fs; % Sampling frequency (Hz)
CC.Tsp = 1/CC.f_samp; % Sampling small time constant (s)
CC.Tpe = CC.PWM + CC.Tcs + CC.Tsp; % Equivalent small time constant (s)

% Dominant time constants
CC.Tn=2*DCDC.L/(2*DCDC.Lr); %FOR WHAT
CC.Ti=2*CC.Tpe; %FOR WHAT

% Controller
CC.Kp=CC.Tn/CC.Ti; %FOR WHAT
CC.Ki=1/CC.Ti; %FOR WHAT
CC.Tp_current=2*sqrt(CC.Ti*CC.Tpe); %FOR WHAT
CC.K_aw=1/CC.Kp; %FOR WHAT

CC.upper=DCDC.Vamp; %FOR WHAT
CC.lower=0; %FOR WHAT

%% Voltage controller
VC.Tp_Vloop = CC.Tsp*2; % CC.Tp_current/2;
VC.Tsp=VC.Tp_Vloop; %FOR WHAT
VC.Ti_Vloop = 8/DCDC.C*VC.Tp_Vloop*VC.Tp_Vloop; %FOR WHAT
VC.Tn_Vloop = 4*VC.Tp_Vloop; %FOR WHAT

VC.Kp = VC.Tn_Vloop/VC.Ti_Vloop; %FOR WHAT
VC.Ki = 1/VC.Ti_Vloop; %FOR WHAT
VC.K_aw=1/VC.Kp; %FOR WHAT
VC.upper=30; %FOR WHAT
VC.lower=0; %FOR WHAT